import os
import cv2
import time
from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler

class Watcher:
    DIRECTORY_TO_WATCH = "Z:/nginxFileServe/PhotoGallery"  # Updated to the PhotoGallery directory

    def __init__(self):
        self.observer = Observer()

    def run(self):
        process_existing_files(self.DIRECTORY_TO_WATCH)
        event_handler = Handler()
        self.observer.schedule(event_handler, self.DIRECTORY_TO_WATCH, recursive=True)
        self.observer.start()
        try:
            while True:
                time.sleep(10)  # Introducing a delay to reduce CPU usage
        except:
            self.observer.stop()
            print("Observer Stopped")
        self.observer.join()

class Handler(FileSystemEventHandler):

    @staticmethod
    def on_any_event(event):
        if event.is_directory or 'thumbs' in event.src_path:
            return None
        elif event.event_type == 'created':
            if event.src_path.lower().endswith(('.mp4', '.png', '.jpg', '.jpeg')):
                print(f"Received created event - {event.src_path}.")
                thumbs_dir = os.path.join(os.path.dirname(event.src_path), 'thumbs')
                if not os.path.exists(thumbs_dir):
                    os.makedirs(thumbs_dir)
                Handler.process_file(event.src_path, thumbs_dir)

    @staticmethod
    def process_file(file_path, thumbs_dir):
        thumb_file_name = os.path.splitext(os.path.basename(file_path))[0] + '.jpg'
        thumb_file_path = os.path.join(thumbs_dir, thumb_file_name)
        if not os.path.exists(thumb_file_path):  # Check if thumbnail already exists
            if file_path.lower().endswith('.mp4'):
                Handler.extract_frame(file_path, thumbs_dir)
            elif file_path.lower().endswith(('.png', '.jpg', '.jpeg')):
                Handler.resize_image(file_path, thumbs_dir)

    @staticmethod
    def extract_frame(video_path, thumbs_dir):
        cap = cv2.VideoCapture(video_path)
        success, image = cap.read()
        if success:
            Handler.add_play_button(image, video_path, thumbs_dir)

    @staticmethod
    def add_play_button(image, video_path, thumbs_dir):
        play_button_image = cv2.imread('Z:/nginxFileServe/trans.png')  # Update with the path to your play button image
        if play_button_image is None:
            print("Play button image not found.")
            return
        play_button_size = 400  # Adjust size as needed
        play_button_image = cv2.resize(play_button_image, (play_button_size, play_button_size))
        h, w = image.shape[:2]
        x, y = (w - play_button_size) // 2, (h - play_button_size) // 2
        image[y:y+play_button_size, x:x+play_button_size] = play_button_image

        # Save the thumbnail as a .jpg file and then rename it to .mp4
        jpg_thumb_path = os.path.join(thumbs_dir, os.path.splitext(os.path.basename(video_path))[0] + '.jpg')
        cv2.imwrite(jpg_thumb_path, image)

        mp4_thumb_path = os.path.join(thumbs_dir, os.path.splitext(os.path.basename(video_path))[0] + '.mp4')
        if not os.path.exists(mp4_thumb_path):  # Check if .mp4 thumbnail already exists
            os.rename(jpg_thumb_path, mp4_thumb_path)
            print(f"Thumbnail saved and renamed to {mp4_thumb_path}")
        else:
            print(f"Thumbnail already exists: {mp4_thumb_path}")

    @staticmethod
    def resize_image(image_path, thumbs_dir):
        img = cv2.imread(image_path)
        if img is None or img.size == 0:
            print(f"Unable to read image or image is empty: {image_path}")
            return

        height, width = img.shape[:2]
        if height < 2 or width < 2:
            print(f"Image dimensions too small to resize: {image_path}")
            return

        try:
            resized_img = cv2.resize(img, (width // 2, height // 2))
            thumb_path = os.path.join(thumbs_dir, os.path.basename(image_path))
            cv2.imwrite(thumb_path, resized_img)
            print(f"Resized image saved at {thumb_path}")
        except cv2.error as e:
            print(f"Error resizing image {image_path}: {e}")

def process_existing_files(directory):
    for root, dirs, files in os.walk(directory):
        if 'thumbs' in root:
            continue
        thumbs_dir = os.path.join(root, 'thumbs')
        if not os.path.exists(thumbs_dir):
            os.makedirs(thumbs_dir)
        for file in files:
            if file.lower().endswith(('.mp4', '.png', '.jpg', '.jpeg')):
                file_path = os.path.join(root, file)
                print(f"Processing existing file: {file_path}")
                Handler.process_file(file_path, thumbs_dir)

if __name__ == '__main__':
    w = Watcher()
    w.run()
